import os
from typing import List

import pytest

from tmpdir_eraser.tmpdir_eraser import *


def mock_get_unused_dirs() -> List[str]:
    """
    Function that mocks tmpdir_eraser.get_unused_dirs()

    :return: Static list of throwaway directories
    """
    dirs = ["test/test1", "test/test2", "test/test3"]
    for d in dirs:
        try:
            os.makedirs(d)
        except FileExistsError:
            pass

    return dirs


def test_erase_dirs():
    """
    Tests that the tool correctly deletes directories
    """
    dirs_to_erase = mock_get_unused_dirs()
    erase_dirs(dirs_to_erase)

    for d in dirs_to_erase:
        assert not os.path.exists(d)


def test_erase_dirs_error(caplog):
    """
    Tests that erase_dirs correctly errors out when attempting to erase a non-existant directory
    """
    erase_dirs(["dir_that_doesnt_exist"])
    assert ("[Errno 2] No such file or directory" in record for record in caplog.records)


def test_get_database_url():
    """
    Tests that the database URL is well-formed
    """
    assert [s in get_database_url() for s in ["postgresql", "nrao.edu/archive"]]


def test_query_for_tmpdirs():
    """
    Tests that the database is properly queried for tmpdirs
    """
    query_for_tmpdirs(connect_to_database())


def test_get_unused_dirs():
    """
    Tests that the function finds all unused dirs
    """
    dirs = mock_get_unused_dirs()
    assert (d in get_unused_dirs("test") for d in dirs)
    erase_dirs(dirs)


def test_list_difference():
    """
    Tests that list_difference correctly calculates the difference between two lists
    """
    assert list_difference([1, 2, 3], [2, 3]) == [1]
    assert list_difference([3], []) == [3]
    assert list_difference([], [1, 2, 3]) == []
    assert list_difference([1], [1, 2, 3]) == []
    assert list_difference(["hello", "there"], [1, 2, 3]) == ["hello", "there"]


def test_connect_to_database(caplog):
    """
    Tests that the DB connection is well-configured
    """
    with pytest.raises(SystemExit):
        connect_to_database("invalid_url")
        assert ("Invalid database URL" in record for record in caplog.records)
