from __future__ import annotations

import json
from pathlib import Path

from pycapo import CapoConfig
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

Base = declarative_base()


class JSONSerializable:
    def __json__(self, request=None) -> dict:
        """
        Allows this object to be converted to JSON

        :param request: this parameter is the active Pyramid request, if applicable (None otherwise)
        :return: a dictionary which can be converted to JSON using json.dump
        """
        pass

    @classmethod
    def from_json(cls, json: dict) -> any:
        pass


class AbstractFile(JSONSerializable):
    """
    Abstract file is exactly that, an abstract concept of what a file is, to be
    returned from various non-filesystem places.
    """

    def __init__(self, filename: str, content: bytes):
        self.filename, self.content = filename, content

    def write_to(self, directory: Path):
        (directory / self.filename).write_bytes(self.content)

    @classmethod
    def from_path(cls, path: Path) -> AbstractFile:
        return cls(path.name, path.read_bytes())


class AbstractRequest(JSONSerializable):
    """
    Abstract File is an abstract concept that is used to create requests
    """

    name: str
    argument: json

    def __init__(self, name: str, argument: json):
        self.name, self.argument = name, argument


def get_engine():
    """
    Generate the SQL Alchemy engine for us, using Capo

    :return: SQLAlchemy engine from database URL
    """

    capo = CapoConfig().settings("metadataDatabase")
    url = capo.jdbcUrl.replace("jdbc:", "").replace(
        "://", f"://{capo.jdbcUsername}:{capo.jdbcPassword}@"
    )
    return create_engine(url)


def get_session_factory(engine):
    """
    Generate the SQL Alchemy session factory for us, using the supplied engine

    :param engine:
    :return: SQLAlchemy engine
    """

    factory = sessionmaker(expire_on_commit=False)
    factory.configure(bind=engine)
    return factory
